library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity PegasusPicoBlaze is
    Port (
        mclk    : in std_logic;
	   R       : out std_logic;
	   G       : out std_logic;
	   B       : out std_logic;
        VS      : out std_logic;
	   HS      : out std_logic;
        btn     : in std_logic_vector(3 downto 0);
        swt     : in std_logic_vector(7 downto 0);
        led     : out std_logic_vector(7 downto 0);
        an      : out std_logic_vector(3 downto 0);
        ssg     : out std_logic_vector(7 downto 0));
end PegasusPicoBlaze;

architecture Behavioral of PegasusPicoBlaze is

	------------------------------------------------------------------------
	-- Components
	------------------------------------------------------------------------

	component embedded_kcpsm is
	    Port (      
	          port_id : out std_logic_vector(7 downto 0);
          write_strobe : out std_logic;
           read_strobe : out std_logic;
	         out_port : out std_logic_vector(7 downto 0);
               in_port : in std_logic_vector(7 downto 0);
             interrupt : in std_logic;
                 reset : in std_logic;
	              clk : in std_logic);
	end component;

	component seven_segment_display is
	    Port (    
	           hex_in : in std_logic_vector(3 downto 0);
             segment_a : out std_logic;
             segment_b : out std_logic;
             segment_c : out std_logic;
             segment_d : out std_logic;
             segment_e : out std_logic;
             segment_f : out std_logic;
             segment_g : out std_logic;
                   clk : in std_logic );
    end component;

    component vga is
    Port (    
           clk : in std_logic;
           res : in std_logic;
		 number : in  std_logic_vector(7 downto 0);
		 control : in  std_logic_vector(7 downto 0);
           HS : out std_logic;
           VS : out std_logic;
           R : out std_logic;
           G : out std_logic;
           B : out std_logic );
    end component;


	------------------------------------------------------------------------
	-- Signals
	------------------------------------------------------------------------

	signal clkdiv  	: std_logic_vector(23 downto 0);
	signal cntr    	: std_logic_vector(3 downto 0);
	signal cclk    	: std_logic;
	signal dig     	: std_logic_vector(6 downto 0);

	signal port_id		: std_logic_vector(7 downto 0);
	signal write_strobe : std_logic;
	signal out_port	: std_logic_vector(7 downto 0);
	signal read_strobe	: std_logic;
	signal in_port		: std_logic_vector(7 downto 0);
	signal interrupt	: std_logic;
	signal reset		: std_logic;

	signal numeric_value: std_logic_vector(3 downto 0);
     signal number : std_logic_vector(7 downto 0);
     signal control : std_logic_vector(7 downto 0);

	------------------------------------------------------------------------
	-- Module Implementation
	------------------------------------------------------------------------

	begin

	processor: embedded_kcpsm
	    port map(      
	    	   port_id 		=> port_id,
	  	   write_strobe 	=> write_strobe,
	    	   out_port 		=> out_port,
	        read_strobe 	=> read_strobe,
	        in_port 		=> in_port,
	        interrupt 		=> interrupt,
	        reset 			=> reset,
             clk 			=> cclk);

    video: vga
    	    port map(      
	    	   clk       		=> mclk,
	  	   res         	=> reset,
	        HS     		=> HS,
	        VS     		=> VS,
		   R       		=> R,
		   G       		=> G,
		   B       		=> B,
             number		     => number,
		   control		=> control);

	numeric_display: seven_segment_display 
	    port map( 
	        hex_in 		=> numeric_value,
             segment_a 		=> dig(0),
             segment_b 		=> dig(1),
             segment_c 		=> dig(2),
             segment_d 		=> dig(3),
             segment_e 		=> dig(4),
             segment_f 		=> dig(5),
             segment_g		=> dig(6),
                   clk 		=> cclk );


	input_ports: process(cclk)
	begin
	  if cclk'event and cclk='1' then
	    case port_id is
	      when "00000000" =>     
	        in_port <= swt;
	      when "11111111" =>    
	        in_port <= ext(btn,8);
	      when others =>    
	    null;
	    end case;
	  end if; 
	end process input_ports;
	
	output_ports: process(cclk)
	begin
	  if cclk'event and cclk='1' then
	    if write_strobe='1' then
	      case port_id is
	        when "00000000" =>    
		     an  <= "1110";
		     numeric_value <= out_port(3 downto 0);
	        when "00000001" =>    
		     an  <= "1101";
		     numeric_value <= out_port(3 downto 0);
	        when "00000010" =>    
		     an  <= "1011";
		     numeric_value <= out_port(3 downto 0);
	        when "00000011" =>    
		     an  <= "0111";
		     numeric_value <= out_port(3 downto 0);
	        when "00001111" => 
		   	number <= out_port;   
		   when "11110000" =>
		   	control <= out_port;
             when "10100000" =>
		     led <= out_port ;
	        when others => null;
	      end case;
	    else
	      an <= "1111";
	    end if;
	  end if; 
	end process output_ports;
	
	ssg <= not dig;

	process (mclk)
	begin
	    if mclk = '1' and mclk'Event then
	      clkdiv <= clkdiv + 1;
	    end if;
	end process;

	cclk <= clkdiv(15);

end Behavioral;

